@@ -71,10 +71,10 @@ class WakaQ:
         self.queues = list(map(lambda q: self._default_priority(q, lowest_priority.priority), self.queues))
         self.queues.sort(key=lambda q: q.priority)
         self.queues_by_name = dict([(x.name, x) for x in self.queues])
-        self.broker_keys = [x.broker_key for x in self.queues]
+        self.exclude_queues = self._validate_queue_names(exclude_queues)
+        self.broker_keys = [x.broker_key for x in self.queues if x.name not in self.exclude_queues]
         self.schedules = [CronTask.create(x) for x in schedules]
         self.concurrency = self._format_concurrency(concurrency)
-        self.exclude_queues = self._validate_queue_names(exclude_queues)
         self.soft_timeout = soft_timeout.total_seconds() if isinstance(soft_timeout, timedelta) else soft_timeout
         self.hard_timeout = hard_timeout.total_seconds() if isinstance(hard_timeout, timedelta) else hard_timeout
         self.wait_timeout = wait_timeout.total_seconds() if isinstance(wait_timeout, timedelta) else wait_timeout
@@ -188,12 +188,10 @@ class WakaQ:
         timestamp = calendar.timegm(eta.utctimetuple())
         self.broker.zadd(queue.broker_eta_key, {payload: timestamp}, nx=True)
 
-    def _broadcast(self, task_name: str, queue: str, args: list, kwargs: dict):
-        queue = self._queue_or_default(queue)
+    def _broadcast(self, task_name: str, args: list, kwargs: dict):
         payload = serialize(
             {
                 "name": task_name,
-                "queue": queue.name,
                 "args": args,
                 "kwargs": kwargs,
             }
