@@ -15,7 +15,7 @@ import psutil
 from .exceptions import SoftTimeout
 from .logger import setup_logging
 from .serializer import deserialize
-from .utils import current_task, kill, read_fd
+from .utils import close_fd, current_task, flush_fh, kill, read_fd, write_fd
 
 ZRANGEPOP = """
 local results = redis.call('ZRANGEBYSCORE', KEYS[1], 0, ARGV[1])
@@ -30,30 +30,30 @@ log = logging.getLogger("wakaq")
 class Child:
     __slots__ = [
         "pid",
-        "pingin",
         "stdin",
+        "pingin",
+        "broadcastout",
         "last_ping",
         "soft_timeout_reached",
         "done",
     ]
 
-    def __init__(self, pid, pingin, stdin):
+    def __init__(self, pid, stdin, pingin, broadcastout):
+        os.set_blocking(stdin, False)
+        os.set_blocking(pingin, False)
+        os.set_blocking(broadcastout, False)
         self.pid = pid
-        self.pingin = pingin
         self.stdin = stdin
+        self.pingin = pingin
+        self.broadcastout = broadcastout
         self.soft_timeout_reached = False
         self.last_ping = time.time()
         self.done = False
 
     def close(self):
-        try:
-            os.close(self.pingin)
-        except:
-            pass
-        try:
-            os.close(self.stdin)
-        except:
-            pass
+        close_fd(self.pingin)
+        close_fd(self.stdin)
+        close_fd(self.broadcastout)
 
 
 class Worker:
@@ -64,6 +64,7 @@ class Worker:
         "_max_mem_reached_at",
         "_pubsub",
         "_pingout",
+        "_broadcastin",
         "_stdout",
         "_num_tasks_processed",
     ]
@@ -97,16 +98,19 @@ class Worker:
 
     def _fork(self) -> int:
         pingin, pingout = os.pipe()
+        broadcastin, broadcastout = os.pipe()
         stdin, stdout = os.pipe()
         pid = os.fork()
         if pid == 0:
-            os.close(pingin)
-            os.close(stdin)
-            self._child(pingout, stdout)
+            close_fd(stdin)
+            close_fd(pingin)
+            close_fd(broadcastout)
+            self._child(stdout, pingout, broadcastin)
         else:
-            os.close(pingout)
-            os.close(stdout)
-            self._add_child(pid, pingin, stdin)
+            close_fd(stdout)
+            close_fd(pingout)
+            close_fd(broadcastin)
+            self._add_child(pid, stdin, pingin, broadcastout)
         return pid
 
     def _parent(self):
@@ -137,23 +141,27 @@ class Worker:
                 self._check_child_runtimes()
                 time.sleep(0.05)
 
-    def _child(self, pingout, stdout):
+    def _child(self, stdout, pingout, broadcastin):
+        os.set_blocking(pingout, False)
+        os.set_blocking(broadcastin, False)
+        os.set_blocking(stdout, False)
         self._pingout = pingout
+        self._broadcastin = broadcastin
         self._stdout = stdout
 
         fh = os.fdopen(stdout, "w")
         sys.stdout = fh
         sys.stderr = fh
 
-        # ignore ctrl-c sent to process group from terminal
-        signal.signal(signal.SIGINT, signal.SIG_IGN)
-
         # reset sigchld
         signal.signal(signal.SIGCHLD, signal.SIG_DFL)
 
         # stop processing and gracefully shutdown
         signal.signal(signal.SIGTERM, self._on_exit_child)
 
+        # ignore ctrl-c sent to process group from terminal
+        signal.signal(signal.SIGINT, signal.SIG_IGN)
+
         # raise SoftTimeout
         signal.signal(signal.SIGQUIT, self._on_soft_timeout_child)
 
@@ -174,25 +182,27 @@ class Worker:
 
             self._num_tasks_processed = 0
             while not self._stop_processing:
-                self._execute_next_task_from_queue()
+                payload = self.wakaq._blocking_dequeue()
+                self._execute_task(payload)
+                self._execute_broadcast_tasks()
                 if self.wakaq.max_tasks_per_worker and self._num_tasks_processed >= self.wakaq.max_tasks_per_worker:
-                    log.debug(f"restarting worker after {self._num_tasks_processed} tasks")
+                    log.info(f"restarting worker after {self._num_tasks_processed} tasks")
                     self._stop_processing = True
-                sys.stdout.flush()
+                flush_fh(sys.stdout)
 
         except:
             log.error(traceback.format_exc())
 
         finally:
-            sys.stdout.flush()
-            sys.stderr.flush()
             sys.stdout = sys.__stdout__
             sys.stderr = sys.__stderr__
-            os.close(self._stdout)
-            os.close(self._pingout)
+            flush_fh(fh)
+            close_fd(self._broadcastin)
+            close_fd(self._stdout)
+            close_fd(self._pingout)
 
-    def _add_child(self, pid, pingin, stdin):
-        self.children.append(Child(pid, pingin, stdin))
+    def _add_child(self, pid, stdin, pingin, broadcastout):
+        self.children.append(Child(pid, stdin, pingin, broadcastout))
 
     def _remove_all_children(self):
         for child in self.children:
@@ -238,8 +248,7 @@ class Worker:
                 kwargs = payload.pop("kwargs")
                 self.wakaq._enqueue_at_front(task_name, queue.name, args, kwargs)
 
-    def _execute_next_task_from_queue(self):
-        payload = self.wakaq._blocking_dequeue()
+    def _execute_task(self, payload):
         if payload is not None:
             task = self.wakaq.tasks[payload["name"]]
             current_task.set(task)
@@ -260,6 +269,14 @@ class Worker:
                     self.wakaq.after_task_finished_callback()
         os.write(self._pingout, b".")
 
+    def _execute_broadcast_tasks(self):
+        payloads = read_fd(self._broadcastin)
+        if payloads == b"":
+            return
+        for payload in payloads.decode("utf8").splitlines():
+            payload = deserialize(payload)
+            self._execute_task(payload)
+
     def _read_child_logs(self):
         for child in self.children:
             logs = read_fd(child.stdin)
@@ -306,12 +323,12 @@ class Worker:
         msg = self._pubsub.get_message(ignore_subscribe_messages=True, timeout=self.wakaq.wait_timeout)
         if msg:
             payload = msg["data"]
-            payload = deserialize(payload)
-            task_name = payload.pop("name")
-            queue = payload.pop("queue")
-            args = payload.pop("args")
-            kwargs = payload.pop("kwargs")
-            self.wakaq._enqueue_at_front(task_name, queue, args, kwargs)
+            for child in self.children:
+                if child.done:
+                    continue
+                log.debug(f"run broadcast task: {payload}")
+                write_fd(child.broadcastout, f"{payload}\n")
+                break
 
     def _refork_missing_children(self):
         if self._stop_processing:
